@@ -15,6 +15,6 @@ class Touching(KinematicsMixin, RelativeObjectState, BooleanState):
         assert ContactBodies in objB_states
 
         for item in objA_states[ContactBodies].get_value():
-            if item.bodyUniqueIdB == other.get_body_id():
+            if item.bodyUniqueIdB in other.get_body_ids():
                 return True
         return False
