@@ -7,18 +7,24 @@ import pybullet as p
 import igibson
 from igibson.external.pybullet_tools import utils
 from igibson.external.pybullet_tools.utils import get_aabb_extent, get_link_name, link_from_name
-from igibson.objects.object_base import Object
+from igibson.objects.object_base import BaseObject
 from igibson.utils import sampling_utils
-from igibson.utils.constants import PyBulletSleepState, SemanticClass
+from igibson.utils.constants import PyBulletSleepState
 
 _STASH_POSITION = [0, 0, -100]
 
 
-class Particle(Object):
+class Particle(BaseObject):
     """
     A particle object, used to simulate water stream and dust/stain
     """
 
+    DEFAULT_RENDERING_PARAMS = {
+        "use_pbr": False,
+        "use_pbr_mapping": False,
+        "shadow_caster": True,
+    }
+
     def __init__(
         self,
         size,
@@ -29,6 +35,7 @@ class Particle(Object):
         base_shape="sphere",
         mesh_filename=None,
         mesh_bounding_box=None,
+        **kwargs
     ):
         """
         Create a particle.
@@ -41,8 +48,9 @@ class Particle(Object):
         :param base_shape: One of "cube", "sphere", "mesh". If mesh, mesh_filename also required.
         :param mesh_filename: Filename of obj file to load mesh from, if base_shape is "mesh".
         :param mesh_bounding_box: bounding box of the mesh when scale=1. Needed for scale computation.
+        :param rendering_params: rendering parameters to pass onto object base & renderer.
         """
-        super(Particle, self).__init__()
+        super(Particle, self).__init__(**kwargs)
         self.base_pos = pos
         self.size = size
         self.visual_only = visual_only
@@ -57,7 +65,7 @@ class Particle(Object):
             self.mesh_filename = mesh_filename
             self.mesh_scale = np.array(size) / np.array(mesh_bounding_box)
 
-    def _load(self):
+    def _load(self, simulator):
         """
         Load the object into pybullet
         """
@@ -89,34 +97,24 @@ class Particle(Object):
 
         p.resetBasePositionAndOrientation(body_id, np.array(self.base_pos), base_orientation)
 
+        simulator.load_object_in_renderer(self, body_id, self.class_id, **self._rendering_params)
+
         self.force_sleep(body_id)
 
-        return body_id
+        return [body_id]
 
     def force_sleep(self, body_id=None):
         if body_id is None:
-            body_id = self.body_id
+            body_id = self.get_body_ids()[0]
 
         activationState = p.ACTIVATION_STATE_SLEEP + p.ACTIVATION_STATE_DISABLE_WAKEUP
         p.changeDynamics(body_id, -1, activationState=activationState)
 
-    def force_wakeup(self):
-        activationState = p.ACTIVATION_STATE_WAKE_UP
-        p.changeDynamics(self.body_id, -1, activationState=activationState)
-
 
 class ParticleSystem(object):
-    def __init__(
-        self,
-        num,
-        size,
-        color=(1, 1, 1, 1),
-        class_id=SemanticClass.USER_ADDED_OBJS,
-        use_pbr=False,
-        use_pbr_mapping=False,
-        shadow_caster=True,
-        **kwargs
-    ):
+    DEFAULT_RENDERING_PARAMS = {}  # Accept the Particle defaults but expose this interface for children
+
+    def __init__(self, num, size, color=(1, 1, 1, 1), rendering_params=None, **kwargs):
         size = np.array(size)
         if size.ndim == 2:
             assert size.shape[0] == num
@@ -131,19 +129,19 @@ class ParticleSystem(object):
         self._particles_activated_at_any_time = set()
 
         self._simulator = None
-        self._import_params = {
-            "class_id": class_id,
-            "use_pbr": use_pbr,
-            "use_pbr_mapping": use_pbr_mapping,
-            "shadow_caster": shadow_caster,
-        }
+
+        rendering_params_for_particle = dict(self.DEFAULT_RENDERING_PARAMS)
+        if rendering_params is not None:
+            rendering_params_for_particle.update(rendering_params)
 
         for i in range(num):
             # If different sizes / colors provided for each instance, pick the correct one for this instance.
             this_size = size if size.ndim == 1 else size[i]
             this_color = color if color.ndim == 1 else color[i]
 
-            particle = Particle(this_size, _STASH_POSITION, color=this_color, **kwargs)
+            particle = Particle(
+                this_size, _STASH_POSITION, color=this_color, rendering_params=rendering_params_for_particle, **kwargs
+            )
             self._all_particles.append(particle)
             self._stashed_particles.append(particle)
 
@@ -207,10 +205,10 @@ class ParticleSystem(object):
             particle.force_sleep()
 
     def _load_particle(self, particle):
-        body_id = self._simulator.import_object(particle, **self._import_params)
+        body_ids = particle.load(self._simulator)
         # Put loaded particles at the stash position initially.
         particle.set_position(_STASH_POSITION)
-        return body_id
+        return body_ids
 
     def unstash_particle(self, position, orientation, particle=None):
         # If the user wants a particular particle, give it to them. Otherwise, unstash one.
@@ -220,7 +218,7 @@ class ParticleSystem(object):
             particle = self._stashed_particles.popleft()
 
         # Lazy loading of the particle now if not already loaded
-        if not particle.loaded:
+        if particle.get_body_ids() is None:
             self._load_particle(particle)
 
         particle.set_position_orientation(position, orientation)
@@ -252,6 +250,17 @@ class AttachedParticleSystem(ParticleSystem):
         super(AttachedParticleSystem, self).__init__(**kwargs)
 
         self.parent_obj = parent_obj
+
+        # TODO: Avoid this logic. This is necessitated by the fact that all of our currently existing scenes are
+        # cached with single-body-attached particles. We don't need this to be true.
+        parent_body_ids = self.parent_obj.get_body_ids()
+        assert parent_body_ids, "Object needs to have a body ID."
+        if len(parent_body_ids) == 1:
+            self.parent_body_id = parent_body_ids[0]
+        else:
+            assert hasattr(self.parent_obj, "main_body"), "The main body ID needs to be annotated on the object."
+            self.parent_body_id = self.parent_obj.get_body_ids()[self.parent_obj.main_body]
+
         self._attachment_offsets = {}  # in the format of {particle: offset}
         self.initial_dump = initial_dump
 
@@ -272,9 +281,7 @@ class AttachedParticleSystem(ParticleSystem):
                 particle_attached_link_id = -1
                 if particle_attached_link_name is not None:
                     try:
-                        particle_attached_link_id = link_from_name(
-                            self.parent_obj.get_body_id(), particle_attached_link_name
-                        )
+                        particle_attached_link_id = link_from_name(self.parent_body_id, particle_attached_link_name)
                     except ValueError:
                         pass
 
@@ -297,7 +304,7 @@ class AttachedParticleSystem(ParticleSystem):
             attachment_source_pos = self.parent_obj.get_position()
             attachment_source_orn = self.parent_obj.get_orientation()
         else:
-            link_state = utils.get_link_state(self.parent_obj.get_body_id(), link_id)
+            link_state = utils.get_link_state(self.parent_body_id, link_id)
             attachment_source_pos = link_state.linkWorldPosition
             attachment_source_orn = link_state.linkWorldOrientation
 
@@ -318,14 +325,14 @@ class AttachedParticleSystem(ParticleSystem):
         for particle in self.get_active_particles():
             link_id, (pos_offset, orn_offset) = self._attachment_offsets[particle]
 
-            dynamics_info = p.getDynamicsInfo(self.parent_obj.get_body_id(), link_id)
+            dynamics_info = p.getDynamicsInfo(self.parent_body_id, link_id)
 
             if len(dynamics_info) == 13:
                 activation_state = dynamics_info[12]
             else:
                 activation_state = PyBulletSleepState.AWAKE
 
-            if activation_state != PyBulletSleepState.AWAKE:
+            if activation_state not in [PyBulletSleepState.AWAKE, PyBulletSleepState.ISLAND_AWAKE]:
                 # If parent object is in sleep, don't update particle poses
                 continue
 
@@ -333,7 +340,7 @@ class AttachedParticleSystem(ParticleSystem):
                 attachment_source_pos = self.parent_obj.get_position()
                 attachment_source_orn = self.parent_obj.get_orientation()
             else:
-                link_state = utils.get_link_state(self.parent_obj.get_body_id(), link_id)
+                link_state = utils.get_link_state(self.parent_body_id, link_id)
                 attachment_source_pos = link_state.linkWorldPosition
                 attachment_source_orn = link_state.linkWorldOrientation
 
@@ -356,8 +363,8 @@ class AttachedParticleSystem(ParticleSystem):
                     attachment_source_pos = self.parent_obj.get_position()
                     attachment_source_orn = self.parent_obj.get_orientation()
                 else:
-                    link_name = get_link_name(self.parent_obj.get_body_id(), link_id)
-                    link_state = utils.get_link_state(self.parent_obj.get_body_id(), link_id)
+                    link_name = get_link_name(self.parent_body_id, link_id)
+                    link_state = utils.get_link_state(self.parent_body_id, link_id)
                     attachment_source_pos = link_state.linkWorldPosition
                     attachment_source_orn = link_state.linkWorldOrientation
 
@@ -371,25 +378,18 @@ class AttachedParticleSystem(ParticleSystem):
 
 class WaterStream(ParticleSystem):
     _DROP_PERIOD = 0.1  # new water every this many seconds.
-    _SIZE_OPTIONS = np.array(
-        [
-            [0.02] * 3,
-            [0.018] * 3,
-            [0.016] * 3,
-        ]
-    )
-    _COLOR_OPTIONS = np.array([(0.61, 0.82, 0.86, 1), (0.5, 0.77, 0.87, 1)])
+    _SIZE = np.array([0.02] * 3)
+    _COLOR = np.array([0.61, 0.82, 0.86, 1])
+    DEFAULT_RENDERING_PARAMS = {"use_pbr": True}  # PBR needs to be on for the shiny water particles.
 
     def __init__(self, water_source_pos, num, initial_dump=None, **kwargs):
+        # Backward compatibility: we no longer randomize the sizes and colors because we want to reload scene with state caches
         if initial_dump is not None:
             self.sizes = np.array(initial_dump["sizes"])
             self.colors = np.array(initial_dump["colors"])
         else:
-            size_idxs = np.random.choice(len(self._SIZE_OPTIONS), num, replace=True)
-            self.sizes = self._SIZE_OPTIONS[size_idxs]
-
-            color_idxs = np.random.choice(len(self._COLOR_OPTIONS), num, replace=True)
-            self.colors = self._COLOR_OPTIONS[color_idxs]
+            self.sizes = np.tile(self._SIZE, (num, 1))
+            self.colors = np.tile(self._COLOR, (num, 1))
 
         super(WaterStream, self).__init__(
             num=num,
@@ -397,7 +397,6 @@ class WaterStream(ParticleSystem):
             color=self.colors,
             visual_only=False,
             mass=0.00005,  # each drop is around 0.05 grams
-            use_pbr=True,  # PBR needs to be on for the shiny water particles.
             **kwargs
         )
 
@@ -407,9 +406,10 @@ class WaterStream(ParticleSystem):
         self.initial_dump = initial_dump
 
     def reset_to_dump(self, dump):
+        # Need to comment out for backward compatibility for existing scene caches
         # Assert that the dump is compatible with the particle system state.
-        assert np.all(self.sizes == np.array(dump["sizes"])), "Incompatible WaterStream dump."
-        assert np.all(self.colors == np.array(dump["colors"])), "Incompatible WaterStream dump."
+        # assert np.all(self.sizes == np.array(dump["sizes"])), "Incompatible WaterStream dump."
+        # assert np.all(self.colors == np.array(dump["colors"])), "Incompatible WaterStream dump."
 
         self.steps_since_last_drop_step = dump["steps_since_last_drop_step"]
 
@@ -431,16 +431,16 @@ class WaterStream(ParticleSystem):
 
     def _load_particle(self, particle):
         # First load the particle normally.
-        body_id = super(WaterStream, self)._load_particle(particle)
+        body_ids = super(WaterStream, self)._load_particle(particle)
 
         # Set renderer instance settings on the particles.
         instances = self._simulator.renderer.get_instances()
         for instance in instances:
-            if instance.pybullet_uuid == body_id:
+            if instance.pybullet_uuid in body_ids:
                 instance.roughness = 0
                 instance.metalness = 1
 
-        return body_id
+        return body_ids
 
     def set_running(self, on):
         self.on = on
@@ -567,6 +567,7 @@ class Stain(_Dirt):
 
     def __init__(self, parent_obj, initial_dump=None, **kwargs):
         if initial_dump:
+            # Backward compatibility: we no longer randomize the bbox dims because we want to reload scene with state caches
             self.random_bbox_dims = np.array(initial_dump["random_bbox_dims"])
         else:
             # Particle size range changes based on parent object size.
@@ -592,9 +593,9 @@ class Stain(_Dirt):
                 self._BOUNDING_BOX_UPPER_LIMIT_MAX,
             )
 
-            # Here we randomize the size of the base (XY plane) of the stain while keeping height constant.
-            random_bbox_base_size = np.random.uniform(
-                bounding_box_lower_limit, bounding_box_upper_limit, self._PARTICLE_COUNT
+            # Fixed but different sizes
+            random_bbox_base_size = np.linspace(
+                bounding_box_lower_limit, bounding_box_upper_limit, num=self._PARTICLE_COUNT, endpoint=True
             )
             self.random_bbox_dims = np.stack(
                 [
@@ -619,8 +620,9 @@ class Stain(_Dirt):
         )
 
     def reset_to_dump(self, dump):
+        # Need to comment out for backward compatibility for existing scene caches
         # Assert that the dump is compatible.
-        assert np.all(np.array(dump["random_bbox_dims"]) == self.random_bbox_dims)
+        # assert np.all(np.array(dump["random_bbox_dims"]) == self.random_bbox_dims)
 
         # Call the dump resetter of the parent.
         super(Stain, self).reset_to_dump(dump["dirt_dump"])
