@@ -1,16 +1,19 @@
+import logging
 from time import sleep, time
 
 import numpy as np
 import pybullet as p
 from transforms3d import euler
 
+log = logging.getLogger(__name__)
+
+
 from igibson.external.pybullet_tools.utils import (
     control_joints,
     get_base_values,
     get_joint_positions,
     get_max_limits,
     get_min_limits,
-    get_moving_links,
     get_sample_fn,
     is_collision_free,
     joints_from_names,
@@ -23,7 +26,7 @@ from igibson.external.pybullet_tools.utils import (
 from igibson.objects.visual_marker import VisualMarker
 from igibson.scenes.gibson_indoor_scene import StaticIndoorScene
 from igibson.scenes.igibson_indoor_scene import InteractiveIndoorScene
-from igibson.utils.utils import l2_distance, quatToXYZW, rotate_vector_2d
+from igibson.utils.utils import l2_distance, quatToXYZW, restoreState, rotate_vector_2d
 
 
 class MotionPlanningWrapper(object):
@@ -31,36 +34,73 @@ class MotionPlanningWrapper(object):
     Motion planner wrapper that supports both base and arm motion
     """
 
-    def __init__(self, env=None, base_mp_algo="birrt", arm_mp_algo="birrt", optimize_iter=0, fine_motion_plan=True):
+    def __init__(
+        self,
+        env=None,
+        base_mp_algo="birrt",
+        arm_mp_algo="birrt",
+        optimize_iter=0,
+        fine_motion_plan=True,
+        full_observability_2d_planning=False,
+        collision_with_pb_2d_planning=False,
+        visualize_2d_planning=False,
+        visualize_2d_result=False,
+    ):
         """
         Get planning related parameters.
         """
         self.env = env
         assert "occupancy_grid" in self.env.output
         # get planning related parameters from env
-        self.robot_id = self.env.robots[0].robot_ids[0]
-        # self.mesh_id = self.scene.mesh_body_id
-        # mesh id should not be used
-        self.map_size = self.env.scene.trav_map_original_size * self.env.scene.trav_map_default_resolution
+        body_ids = self.env.robots[0].get_body_ids()
+        assert len(body_ids) == 1, "Only single-body robots are supported."
+        self.robot_id = body_ids[0]
+
+        # Types of 2D planning
+        # full_observability_2d_planning=TRUE and collision_with_pb_2d_planning=TRUE -> We teleport the robot to locations and check for collisions
+        # full_observability_2d_planning=TRUE and collision_with_pb_2d_planning=FALSE -> We use the global occupancy map from the scene
+        # full_observability_2d_planning=FALSE and collision_with_pb_2d_planning=FALSE -> We use the occupancy_grid from the lidar sensor
+        # full_observability_2d_planning=FALSE and collision_with_pb_2d_planning=TRUE -> [not suported yet]
+        self.full_observability_2d_planning = full_observability_2d_planning
+        self.collision_with_pb_2d_planning = collision_with_pb_2d_planning
+        assert not ((not self.full_observability_2d_planning) and self.collision_with_pb_2d_planning)
 
-        self.grid_resolution = self.env.grid_resolution
-        self.occupancy_range = self.env.sensors["scan_occ"].occupancy_range
         self.robot_footprint_radius = self.env.sensors["scan_occ"].robot_footprint_radius
-        self.robot_footprint_radius_in_map = self.env.sensors["scan_occ"].robot_footprint_radius_in_map
+        if self.full_observability_2d_planning:
+            # TODO: it may be better to unify and make that scene.floor_map uses OccupancyGridState values always
+            assert len(self.env.scene.floor_map) == 1  # We assume there is only one floor (not true for Gibson scenes)
+            self.map_2d = np.array(self.env.scene.floor_map[0])
+            self.map_2d = np.array((self.map_2d == 255)).astype(np.float32)
+            self.per_pixel_resolution = self.env.scene.trav_map_resolution
+            assert np.array(self.map_2d).shape[0] == np.array(self.map_2d).shape[1]
+            self.grid_resolution = self.map_2d.shape[0]
+            self.occupancy_range = self.grid_resolution * self.per_pixel_resolution
+            self.robot_footprint_radius_in_map = int(np.ceil(self.robot_footprint_radius / self.per_pixel_resolution))
+        else:
+            self.grid_resolution = self.env.grid_resolution
+            self.occupancy_range = self.env.sensors["scan_occ"].occupancy_range
+            self.robot_footprint_radius_in_map = self.env.sensors["scan_occ"].robot_footprint_radius_in_map
+
         self.robot = self.env.robots[0]
         self.base_mp_algo = base_mp_algo
         self.arm_mp_algo = arm_mp_algo
-        self.base_mp_resolutions = np.array([0.05, 0.05, 0.05])
+        # If we plan in the map, we do not need to check rotations: a location is in collision (or not) independently
+        # of the orientation. If we use pybullet, we may find some cases where the base orientation changes the
+        # collision value for the same location between True/False
+        if not self.collision_with_pb_2d_planning:
+            self.base_mp_resolutions = np.array([0.05, 0.05, 2 * np.pi])
+        else:
+            self.base_mp_resolutions = np.array([0.05, 0.05, 0.05])
         self.optimize_iter = optimize_iter
         self.mode = self.env.mode
         self.initial_height = self.env.initial_pos_z_offset
         self.fine_motion_plan = fine_motion_plan
-        self.robot_type = self.env.config["robot"]
+        self.robot_type = self.robot.model_name
 
         if self.env.simulator.viewer is not None:
             self.env.simulator.viewer.setup_motion_planner(self)
 
-        if self.robot_type in ["Fetch", "Movo"]:
+        if self.robot_type in ["Fetch"]:
             self.setup_arm_mp()
 
         self.arm_interaction_length = 0.2
@@ -68,7 +108,7 @@ class MotionPlanningWrapper(object):
         self.marker = None
         self.marker_direction = None
 
-        if self.mode in ["gui", "iggui"]:
+        if self.mode in ["gui_non_interactive", "gui_interactive"]:
             self.marker = VisualMarker(radius=0.04, rgba_color=[0, 0, 1, 1])
             self.marker_direction = VisualMarker(
                 visual_shape=p.GEOM_CAPSULE,
@@ -77,8 +117,11 @@ class MotionPlanningWrapper(object):
                 initial_offset=[0, 0, -0.1],
                 rgba_color=[0, 0, 1, 1],
             )
-            self.env.simulator.import_object(self.marker, use_pbr=False)
-            self.env.simulator.import_object(self.marker_direction, use_pbr=False)
+            self.env.simulator.import_object(self.marker)
+            self.env.simulator.import_object(self.marker_direction)
+
+        self.visualize_2d_planning = visualize_2d_planning
+        self.visualize_2d_result = visualize_2d_result
 
     def set_marker_position(self, pos):
         """
@@ -115,56 +158,49 @@ class MotionPlanningWrapper(object):
         """
         if self.robot_type == "Fetch":
             self.arm_default_joint_positions = (
-                0.10322468280792236,
-                -1.414019864768982,
-                1.5178184935241699,
-                0.8189625336474915,
-                2.200358942909668,
-                2.9631312579803466,
-                -1.2862852996643066,
-                0.0008453550418615341,
+                0.1,
+                -1.41,
+                1.517,
+                0.82,
+                2.2,
+                2.96,
+                -1.286,
+                0.0,
             )
+            self.arm_joint_names = [
+                "torso_lift_joint",
+                "shoulder_pan_joint",
+                "shoulder_lift_joint",
+                "upperarm_roll_joint",
+                "elbow_flex_joint",
+                "forearm_roll_joint",
+                "wrist_flex_joint",
+                "wrist_roll_joint",
+            ]
+            self.robot_joint_names = [
+                "r_wheel_joint",
+                "l_wheel_joint",
+                "torso_lift_joint",
+                "head_pan_joint",
+                "head_tilt_joint",
+                "shoulder_pan_joint",
+                "shoulder_lift_joint",
+                "upperarm_roll_joint",
+                "elbow_flex_joint",
+                "forearm_roll_joint",
+                "wrist_flex_joint",
+                "wrist_roll_joint",
+                "r_gripper_finger_joint",
+                "l_gripper_finger_joint",
+            ]
             self.arm_joint_ids = joints_from_names(
                 self.robot_id,
-                [
-                    "torso_lift_joint",
-                    "shoulder_pan_joint",
-                    "shoulder_lift_joint",
-                    "upperarm_roll_joint",
-                    "elbow_flex_joint",
-                    "forearm_roll_joint",
-                    "wrist_flex_joint",
-                    "wrist_roll_joint",
-                ],
+                self.arm_joint_names,
             )
-        elif self.robot_type == "Movo":
-            self.arm_default_joint_positions = (
-                0.205,
-                -1.50058731470836,
-                -1.3002625076695704,
-                0.5204845864369407,
-                -2.6923805472917626,
-                -0.02678584326934146,
-                0.5065742552588746,
-                -1.562883631882778,
-            )
-            self.arm_joint_ids = joints_from_names(
-                self.robot_id,
-                [
-                    "linear_joint",
-                    "right_shoulder_pan_joint",
-                    "right_shoulder_lift_joint",
-                    "right_arm_half_joint",
-                    "right_elbow_joint",
-                    "right_wrist_spherical_1_joint",
-                    "right_wrist_spherical_2_joint",
-                    "right_wrist_3_joint",
-                ],
-            )
-        self.arm_joint_ids_all = get_moving_links(self.robot_id, self.arm_joint_ids)
-        self.arm_joint_ids_all = [
-            item for item in self.arm_joint_ids_all if item != self.robot.end_effector_part_index()
-        ]
+            self.robot_arm_indices = [
+                self.robot_joint_names.index(arm_joint_name) for arm_joint_name in self.arm_joint_names
+            ]
+
         self.arm_ik_threshold = 0.05
 
         self.mp_obstacles = []
@@ -173,6 +209,9 @@ class MotionPlanningWrapper(object):
                 self.mp_obstacles.append(self.env.scene.mesh_body_id)
         elif type(self.env.scene) == InteractiveIndoorScene:
             self.mp_obstacles.extend(self.env.scene.get_body_ids())
+            # Since the refactoring, the robot is another object in the scene
+            # We need to remove it to not check twice for self collisions
+            self.mp_obstacles.remove(self.robot_id)
 
     def plan_base_motion(self, goal):
         """
@@ -184,36 +223,68 @@ class MotionPlanningWrapper(object):
         if self.marker is not None:
             self.set_marker_position_yaw([goal[0], goal[1], 0.05], goal[2])
 
+        log.debug("Motion planning base goal: {}".format(goal))
+
         state = self.env.get_state()
         x, y, theta = goal
-        grid = state["occupancy_grid"]
 
-        yaw = self.robot.get_rpy()[2]
-        half_occupancy_range = self.occupancy_range / 2.0
-        robot_position_xy = self.robot.get_position()[:2]
-        corners = [
-            robot_position_xy + rotate_vector_2d(local_corner, -yaw)
-            for local_corner in [
-                np.array([half_occupancy_range, half_occupancy_range]),
-                np.array([half_occupancy_range, -half_occupancy_range]),
-                np.array([-half_occupancy_range, half_occupancy_range]),
-                np.array([-half_occupancy_range, -half_occupancy_range]),
+        map_2d = state["occupancy_grid"] if not self.full_observability_2d_planning else self.map_2d
+
+        if not self.full_observability_2d_planning:
+            yaw = self.robot.get_rpy()[2]
+            half_occupancy_range = self.occupancy_range / 2.0
+            robot_position_xy = self.robot.get_position()[:2]
+            corners = [
+                robot_position_xy + rotate_vector_2d(local_corner, -yaw)
+                for local_corner in [
+                    np.array([half_occupancy_range, half_occupancy_range]),
+                    np.array([half_occupancy_range, -half_occupancy_range]),
+                    np.array([-half_occupancy_range, half_occupancy_range]),
+                    np.array([-half_occupancy_range, -half_occupancy_range]),
+                ]
             ]
-        ]
+        else:
+            top_left = self.env.scene.map_to_world(np.array([0, 0]))
+            bottom_right = self.env.scene.map_to_world(np.array(self.map_2d.shape) - np.array([1, 1]))
+            corners = [top_left, bottom_right]
+
+        if self.collision_with_pb_2d_planning:
+            obstacles = [
+                body_id
+                for body_id in self.env.scene.get_body_ids()
+                if body_id not in self.robot.get_body_ids()
+                and body_id != self.env.scene.objects_by_category["floors"][0].get_body_ids()[0]
+            ]
+        else:
+            obstacles = []
+
         path = plan_base_motion_2d(
             self.robot_id,
             [x, y, theta],
             (tuple(np.min(corners, axis=0)), tuple(np.max(corners, axis=0))),
-            map_2d=grid,
+            map_2d=map_2d,
             occupancy_range=self.occupancy_range,
             grid_resolution=self.grid_resolution,
-            robot_footprint_radius_in_map=self.robot_footprint_radius_in_map,
+            # If we use the global map, it has been eroded: we do not need to use the full size of the robot, a 1 px
+            # robot would be enough
+            robot_footprint_radius_in_map=[self.robot_footprint_radius_in_map, 1][self.full_observability_2d_planning],
             resolutions=self.base_mp_resolutions,
-            obstacles=[],
+            # Add all objects in the scene as obstacles except the robot itself and the floor
+            obstacles=obstacles,
             algorithm=self.base_mp_algo,
             optimize_iter=self.optimize_iter,
+            visualize_planning=self.visualize_2d_planning,
+            visualize_result=self.visualize_2d_result,
+            metric2map=[None, self.env.scene.world_to_map][self.full_observability_2d_planning],
+            flip_vertically=self.full_observability_2d_planning,
+            use_pb_for_collisions=self.collision_with_pb_2d_planning,
         )
 
+        if path is not None and len(path) > 0:
+            log.debug("Path found!")
+        else:
+            log.debug("Path NOT found!")
+
         return path
 
     def simulator_sync(self):
@@ -232,7 +303,7 @@ class MotionPlanningWrapper(object):
         :param path: base waypoints or None if no plan can be found
         """
         if path is not None:
-            if self.mode in ["gui", "iggui", "pbgui"]:
+            if self.mode in ["gui_non_interactive", "gui_interactive"]:
                 for way_point in path:
                     set_base_values_with_z(
                         self.robot_id, [way_point[0], way_point[1], way_point[2]], z=self.initial_height
@@ -250,19 +321,14 @@ class MotionPlanningWrapper(object):
         """
         max_limits, min_limits, rest_position, joint_range, joint_damping = None, None, None, None, None
         if self.robot_type == "Fetch":
-            max_limits = [0.0, 0.0] + get_max_limits(self.robot_id, self.arm_joint_ids)
-            min_limits = [0.0, 0.0] + get_min_limits(self.robot_id, self.arm_joint_ids)
+            max_limits_arm = get_max_limits(self.robot_id, self.arm_joint_ids)
+            max_limits = [0.5, 0.5] + [max_limits_arm[0]] + [0.5, 0.5] + list(max_limits_arm[1:]) + [0.05, 0.05]
+            min_limits_arm = get_min_limits(self.robot_id, self.arm_joint_ids)
+            min_limits = [-0.5, -0.5] + [min_limits_arm[0]] + [-0.5, -0.5] + list(min_limits_arm[1:]) + [0.0, 0.0]
             # increase torso_lift_joint lower limit to 0.02 to avoid self-collision
             min_limits[2] += 0.02
-            rest_position = [0.0, 0.0] + list(get_joint_positions(self.robot_id, self.arm_joint_ids))
-            joint_range = list(np.array(max_limits) - np.array(min_limits))
-            joint_range = [item + 1 for item in joint_range]
-            joint_damping = [0.1 for _ in joint_range]
-
-        elif self.robot_type == "Movo":
-            max_limits = get_max_limits(self.robot_id, self.robot.all_joints)
-            min_limits = get_min_limits(self.robot_id, self.robot.all_joints)
-            rest_position = list(get_joint_positions(self.robot_id, self.robot.all_joints))
+            current_position = get_joint_positions(self.robot_id, self.arm_joint_ids)
+            rest_position = [0.0, 0.0] + [current_position[0]] + [0.0, 0.0] + list(current_position[1:]) + [0.01, 0.01]
             joint_range = list(np.array(max_limits) - np.array(min_limits))
             joint_range = [item + 1 for item in joint_range]
             joint_damping = [0.1 for _ in joint_range]
@@ -277,6 +343,7 @@ class MotionPlanningWrapper(object):
         :param arm_ik_goal: [x, y, z] in the world frame
         :return: arm joint positions
         """
+        log.debug("IK query for EE position {}".format(arm_ik_goal))
         ik_start = time()
 
         max_limits, min_limits, rest_position, joint_range, joint_damping = self.get_ik_parameters()
@@ -289,13 +356,11 @@ class MotionPlanningWrapper(object):
         # p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, False)
         # find collision-free IK solution for arm_subgoal
         while n_attempt < max_attempt:
-            if self.robot_type == "Movo":
-                self.robot.tuck()
 
             set_joint_positions(self.robot_id, self.arm_joint_ids, sample_fn())
             arm_joint_positions = p.calculateInverseKinematics(
                 self.robot_id,
-                self.robot.end_effector_part_index(),
+                self.robot.eef_links[self.robot.default_arm].link_id,
                 targetPosition=arm_ik_goal,
                 # targetOrientation=self.robots[0].get_orientation(),
                 lowerLimits=min_limits,
@@ -303,18 +368,16 @@ class MotionPlanningWrapper(object):
                 jointRanges=joint_range,
                 restPoses=rest_position,
                 jointDamping=joint_damping,
-                solver=p.IK_DLS,
+                # solver=p.IK_DLS,
                 maxNumIterations=100,
             )
 
             if self.robot_type == "Fetch":
-                arm_joint_positions = arm_joint_positions[2:10]
-            elif self.robot_type == "Movo":
-                arm_joint_positions = arm_joint_positions[:8]
+                arm_joint_positions = np.array(arm_joint_positions)[self.robot_arm_indices]
 
             set_joint_positions(self.robot_id, self.arm_joint_ids, arm_joint_positions)
 
-            dist = l2_distance(self.robot.get_end_effector_position(), arm_ik_goal)
+            dist = l2_distance(self.robot.get_eef_position(), arm_ik_goal)
             # print('dist', dist)
             if dist > self.arm_ik_threshold:
                 n_attempt += 1
@@ -329,11 +392,7 @@ class MotionPlanningWrapper(object):
             # TODO: have a princpled way for stashing and resetting object states
 
             # arm should not have any collision
-            if self.robot_type == "Movo":
-                collision_free = is_collision_free(body_a=self.robot_id, link_a_list=self.arm_joint_ids_all)
-                # ignore linear link
-            else:
-                collision_free = is_collision_free(body_a=self.robot_id, link_a_list=self.arm_joint_ids)
+            collision_free = is_collision_free(body_a=self.robot_id, link_a_list=self.arm_joint_ids)
 
             if not collision_free:
                 n_attempt += 1
@@ -342,23 +401,27 @@ class MotionPlanningWrapper(object):
 
             # gripper should not have any self-collision
             collision_free = is_collision_free(
-                body_a=self.robot_id, link_a_list=[self.robot.end_effector_part_index()], body_b=self.robot_id
+                body_a=self.robot_id,
+                link_a_list=[self.robot.eef_links[self.robot.default_arm].link_id],
+                body_b=self.robot_id,
             )
             if not collision_free:
                 n_attempt += 1
-                print("gripper has collision")
+                log.debug("Gripper in collision")
                 continue
 
             # self.episode_metrics['arm_ik_time'] += time() - ik_start
             # p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, True)
-            p.restoreState(state_id)
+            restoreState(state_id)
             p.removeState(state_id)
+            log.debug("IK Solver found a valid configuration")
             return arm_joint_positions
 
         # p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, True)
-        p.restoreState(state_id)
+        restoreState(state_id)
         p.removeState(state_id)
         # self.episode_metrics['arm_ik_time'] += time() - ik_start
+        log.debug("IK Solver failed to find a configuration")
         return None
 
     def plan_arm_motion(self, arm_joint_positions):
@@ -369,6 +432,7 @@ class MotionPlanningWrapper(object):
         :param arm_joint_positions: final arm joint position to reach
         :return: arm trajectory or None if no plan can be found
         """
+        log.debug("Planning path in joint space to {}".format(arm_joint_positions))
         disabled_collisions = {}
         if self.robot_type == "Fetch":
             disabled_collisions = {
@@ -378,73 +442,6 @@ class MotionPlanningWrapper(object):
                 (link_from_name(self.robot_id, "torso_lift_link"), link_from_name(self.robot_id, "forearm_roll_link")),
                 (link_from_name(self.robot_id, "torso_lift_link"), link_from_name(self.robot_id, "elbow_flex_link")),
             }
-        elif self.robot_type == "Movo":
-            disabled_collisions = {
-                (
-                    link_from_name(self.robot_id, "linear_actuator_link"),
-                    link_from_name(self.robot_id, "right_shoulder_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "right_base_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "linear_actuator_link"),
-                    link_from_name(self.robot_id, "right_arm_half_1_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "linear_actuator_link"),
-                    link_from_name(self.robot_id, "right_arm_half_2_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "linear_actuator_link"),
-                    link_from_name(self.robot_id, "right_forearm_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "linear_actuator_link"),
-                    link_from_name(self.robot_id, "right_wrist_spherical_1_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "linear_actuator_link"),
-                    link_from_name(self.robot_id, "right_wrist_spherical_2_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "linear_actuator_link"),
-                    link_from_name(self.robot_id, "right_wrist_3_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "right_wrist_spherical_2_link"),
-                    link_from_name(self.robot_id, "right_robotiq_coupler_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "right_shoulder_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "left_base_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "left_shoulder_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "left_arm_half_2_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "right_arm_half_2_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "right_arm_half_1_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-                (
-                    link_from_name(self.robot_id, "left_arm_half_1_link"),
-                    link_from_name(self.robot_id, "linear_actuator_fixed_link"),
-                ),
-            }
 
         if self.fine_motion_plan:
             self_collisions = True
@@ -459,10 +456,9 @@ class MotionPlanningWrapper(object):
 
         allow_collision_links = []
         if self.robot_type == "Fetch":
-            allow_collision_links = [19]
-        elif self.robot_type == "Movo":
-            allow_collision_links = [23, 24]
-
+            allow_collision_links = [self.robot.eef_links[self.robot.default_arm].link_id] + [
+                finger.link_id for finger in self.robot.finger_links[self.robot.default_arm]
+            ]
         arm_path = plan_joint_motion(
             self.robot_id,
             self.arm_joint_ids,
@@ -474,8 +470,14 @@ class MotionPlanningWrapper(object):
             allow_collision_links=allow_collision_links,
         )
         p.configureDebugVisualizer(p.COV_ENABLE_RENDERING, True)
-        p.restoreState(state_id)
+        restoreState(state_id)
         p.removeState(state_id)
+
+        if arm_path is not None and len(arm_path) > 0:
+            log.debug("Path found!")
+        else:
+            log.debug("Path NOT found!")
+
         return arm_path
 
     def dry_run_arm_plan(self, arm_path):
@@ -486,7 +488,7 @@ class MotionPlanningWrapper(object):
         """
         base_pose = get_base_values(self.robot_id)
         if arm_path is not None:
-            if self.mode in ["gui", "iggui", "pbgui"]:
+            if self.mode in ["gui_non_interactive", "gui_interactive"]:
                 for joint_way_point in arm_path:
                     set_joint_positions(self.robot_id, self.arm_joint_ids, joint_way_point)
                     set_base_values_with_z(self.robot_id, base_pose, z=self.initial_height)
@@ -495,9 +497,6 @@ class MotionPlanningWrapper(object):
             else:
                 set_joint_positions(self.robot_id, self.arm_joint_ids, arm_path[-1])
         else:
-            # print('arm mp fails')
-            if self.robot_type == "Movo":
-                self.robot.tuck()
             set_joint_positions(self.robot_id, self.arm_joint_ids, self.arm_default_joint_positions)
 
     def plan_arm_push(self, hit_pos, hit_normal):
@@ -508,17 +507,21 @@ class MotionPlanningWrapper(object):
         :param hit_normal: direction to push after reacehing that position
         :return: arm trajectory or None if no plan can be found
         """
+        log.debug("Planning arm push at point {} with direction {}".format(hit_pos, hit_normal))
         if self.marker is not None:
             self.set_marker_position_direction(hit_pos, hit_normal)
+
+        # Solve the IK problem to set the arm at the desired position
         joint_positions = self.get_arm_joint_positions(hit_pos)
 
-        # print('planned JP', joint_positions)
-        set_joint_positions(self.robot_id, self.arm_joint_ids, self.arm_default_joint_positions)
-        self.simulator_sync()
         if joint_positions is not None:
+            # Set the arm in the default configuration to initiate arm motion planning (e.g. untucked)
+            set_joint_positions(self.robot_id, self.arm_joint_ids, self.arm_default_joint_positions)
+            self.simulator_sync()
             plan = self.plan_arm_motion(joint_positions)
             return plan
         else:
+            log.debug("Planning failed: goal position may be non-reachable")
             return None
 
     def interact(self, push_point, push_direction):
@@ -540,7 +543,7 @@ class MotionPlanningWrapper(object):
 
             joint_positions = p.calculateInverseKinematics(
                 self.robot_id,
-                self.robot.end_effector_part_index(),
+                self.robot.eef_links[self.robot.default_arm].link_id,
                 targetPosition=push_goal,
                 # targetOrientation=self.robots[0].get_orientation(),
                 lowerLimits=min_limits,
@@ -548,27 +551,21 @@ class MotionPlanningWrapper(object):
                 jointRanges=joint_range,
                 restPoses=rest_position,
                 jointDamping=joint_damping,
-                solver=p.IK_DLS,
+                # solver=p.IK_DLS,
                 maxNumIterations=100,
             )
 
             if self.robot_type == "Fetch":
-                joint_positions = joint_positions[2:10]
-            elif self.robot_type == "Movo":
-                joint_positions = joint_positions[:8]
+                joint_positions = np.array(joint_positions)[self.robot_arm_indices]
 
             control_joints(self.robot_id, self.arm_joint_ids, joint_positions)
 
             # set_joint_positions(self.robot_id, self.arm_joint_ids, joint_positions)
-            achieved = self.robot.get_end_effector_position()
-            # print('ee delta', np.array(achieved) - push_goal, np.linalg.norm(np.array(achieved) - push_goal))
-
-            # if self.robot_type == 'Movo':
-            #    self.robot.control_tuck_left()
+            achieved = self.robot.get_eef_position()
             self.simulator_step()
             set_base_values_with_z(self.robot_id, base_pose, z=self.initial_height)
 
-            if self.mode in ["pbgui", "iggui", "gui"]:
+            if self.mode == "gui_interactive":
                 sleep(0.02)  # for visualization
 
     def execute_arm_push(self, plan, hit_pos, hit_normal):
@@ -581,7 +578,10 @@ class MotionPlanningWrapper(object):
         :param hit_normal: direction to push after reacehing that position
         """
         if plan is not None:
+            log.debug("Teleporting arm along the trajectory. No physics simulation")
             self.dry_run_arm_plan(plan)
+            log.debug("Performing pushing actions")
             self.interact(hit_pos, hit_normal)
+            log.debug("Teleporting arm to the default configuration")
             set_joint_positions(self.robot_id, self.arm_joint_ids, self.arm_default_joint_positions)
             self.simulator_sync()
